C--------------------------------------------------------------------
C The user of the OPEN-ADAS system and data (hereinafter "the User")
C accepts the following terms and conditions: 
C 
C   1. The University of Strathclyde uses all reasonable endeavours
C      to ensure the accuracy of the data provided and any
C      information given, but the University makes no warranty,
C      expressed or implied as to its accuracy, integrity, fitness
C      for purpose and shall not be responsible for the use to
C      which the data is put by the User and/or any consequences
C      arising out of any inaccuracies or omissions.
C 
C   2. Any downloaded OPEN-ADAS file is made available here for the
C      personal use of the User only.  It must not be used for any
C      commercial application, incorporated into any web site or in
C      any form re-distributed without express prior written
C      permission from the ADAS Project. In particular, but not by
C      way of limitation, it must not be:
C 
C        * inserted into a managed database structure
C        * redistributed along with a modelling or analysis code
C        * made available on a public website.
C 
C Permission for OPEN-ADAS data incorporation in non-commercial
C code/system development for scientific advance will not be
C unreasonably withheld, but will be subject to written agreement
C on a case by case basis.
C 
C The User will indemnify the University of Strathclyde and keep
C it fully and effectively indemnified against each and every
C claim made against the University as a result of the User's
C use of the data.  
C--------------------------------------------------------------------
       PROGRAM TEST
       
       IMPLICIT NONE
C------------------------------------------------------------------------
C  Purpose:
C          Test the XXDATA_08 routine and also provide a brief
C          example of its use.
C
C  Explanaton:
C          This program is distributed as part of the ADAS
C          subroutine library either via direct CVS to ADAS Project
C          members or via download by OPEN-ADAS users. It reads a
C          file called "test.dat" (distributed along with this
C          program) and prints out some of the contents. The
C          program is intended to serve as an example of using the
C          XXDATA_08 subroutine as well as testing it. A short
C          shell script called "test.sh" is used to compile and
C          test the code.
C
C  Variables:
C          All variables are documented by XXDATA_08. See either the
C          Fortran source or the PDF documentation for the routine.
C          Variables not passed into XXDATA_08 are trivial in nature.
C
C  Routines:
C          Routine    Source    Brief description
C          -------------------------------------------------------------
C          XXDATA_08  ADAS      Read complete data from an ADF08 file
C
C  Author : Allan Whiteford
C
C  Date   : 27/03/08
C       
c-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Parameters required by XXDATA_08
C     --------------------------------        
      INTEGER    NDPRT       , NDLEV        , NDT
      PARAMETER( NDPRT = 10  , NDLEV = 1000 , NDT = 14 ) 
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Input variables (i.e. non-parameter) required by XXDATA_08
C     ----------------------------------------------------------
      INTEGER   IUNIT
C-----------------------------------------------------------------------		

C-----------------------------------------------------------------------
C     Output variables required by XXDATA_08
C     --------------------------------------
      INTEGER             IZ,          IZ0,         IZ1
      INTEGER             NPRNT,       NPRNTI,      NTE
      INTEGER             IA(NDLEV),   IL,          ILA(NDLEV)
      INTEGER             ILPA(NDPRT), IPA(NDPRT),  IPRTI(NDPRT)
      INTEGER             ISA(NDLEV),  ISPA(NDPRT), ISPRTI(NDPRT)
      REAL*8              BWNP,        BWNR
      REAL*8              RADR(NDLEV,NDPRT,NDT),    TEA(NDT)
      REAL*8              WA(NDLEV),   WPA(NDPRT),  XJA(NDLEV)
      REAL*8              XJPA(NDPRT)
      CHARACTER*80        CSTRGA(NDLEV),            CSTRPA(NDPRT)
      CHARACTER*2         SEQSYM
      CHARACTER*5         TPRTI(NDPRT)
      LOGICAL             LRADR(NDLEV,NDPRT)
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Variables only used by this test program
C     ----------------------------------------
      CHARACTER*80 DSNIN     
      INTEGER   I
C----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Open input file for reading.
C     ----------------------------
      IUNIT = 10
      DSNIN= 'test.dat'
      OPEN(UNIT = IUNIT , FILE=DSNIN , STATUS = 'OLD')
C-----------------------------------------------------------------------

C-----------------------------------------------------------------------
C     Pass unit number and dimension parameters to XXDATA_08 and
C     get back contents of file.
C     -------------------------- 

      CALL XXDATA_08( IUNIT  , NDPRT  , NDLEV , NDT   ,
     &                SEQSYM , IZ     , IZ0   , IZ1   ,
     &                NPRNT  , NPRNTI , BWNP  ,
     &                IPA    , CSTRPA , ISPA  , ILPA  , XJPA  ,
     &                WPA    ,
     &                IL     , BWNR   ,
     &                IA     , CSTRGA , ISA   , ILA   , XJA   ,
     &                WA     ,
     &                IPRTI  , TPRTI  , ISPRTI, RADR  , LRADR ,
     &                NTE    , TEA
     &              )
C-----------------------------------------------------------------------
   
C-----------------------------------------------------------------------
C     Print out some of the contents to screen
C     ----------------------------------------

      PRINT *,'XXDATA_08 Test Program'      
      PRINT *,'----------------------'
      PRINT *,' '
      PRINT *,'The following is some information about the'
      PRINT *,'input file: '//DSNIN
      PRINT *,' '       

      PRINT *,'PARENT STATES'       
      DO 10 I=1,NPRNT
10        PRINT 1000,I,CSTRPA(I),ISPA(I),ILPA(I),XJPA(I),WPA(I)

      PRINT *,' '       
      PRINT *,'RECOMBINED STATES'       
      DO 20 I=1,IL
20        PRINT 1000,I,CSTRGA(I),ISA(I),ILA(I),XJA(I),WA(I)

      PRINT *,' '       
      PRINT *,'TEMPERATURE GRID'
      PRINT *,(TEA(I),I=1,NTE)

      PRINT *,' '       
      PRINT *,'RECOMBINATION RATE FROM PARENT 2 TO STATE 18 AS'
      PRINT *,'A FUNCTION OF ABOVE TEMPERATURE GRID'
      PRINT *,(RADR(18,2,I),I=1,NTE)

1000  FORMAT(I2,':',1X,A20,'(',I1,')',I1,'(',F4.1,')',F10.1)
C-----------------------------------------------------------------------
       END		
	
